#include <iostream>
#include "AlgoClassiquesLex.h"
#include <sys/types.h>
#include <unistd.h>

// algorithme de Buchberger + optimisations classiques

// constructeur: initialisation de G et S
AlgoClassiquesLex::AlgoClassiquesLex(const std::vector<PolPlusVarLex>& tabPol) {
	nbIter=0;
	Paire paire;
	for (int i=0; i<(int)tabPol.size(); i++) {
		G.push_back(tabPol[i]);
		paire.p1=PolPlusVarLex(tabPol[i]);
		for (int j=i-1; j>=0; j--) {
			paire.p2=PolPlusVarLex(tabPol[j]);
			ajoutePaire(paire);
		}
	}
}

// fonction d'affichage de G
void AlgoClassiquesLex::affiche() {
	for (int i=(int)G.size()-1; i>=0; i--)
		std::cout << G[i] << std::endl;
	return;
}

// affichage sous forme d'un systeme d'equations a satisfaire
void AlgoClassiquesLex::afficheEqualZero() {
	for (int i=(int)G.size()-1; i>=0; i--)
		std::cout << G[i] << " = 0" << std::endl;
	return;
}

// teste si une paire est inutile ou triviale
bool AlgoClassiquesLex::testInutile(const Paire& paire) const {
	if (paire.p1.getSize()==0 || paire.p2.getSize()==0) return true;
	if ((paire.p1[0]).prem(paire.p2[0])) return true;
	TermLex aux=(paire.p1[0]).ppcm(paire.p2[0]);
	Paire p,pp;
	p.p1=paire.p1;
	pp.p2=paire.p2;
	for (int i=G.size()-1; i>=0; i--) {
		p.p2=G[i];
		pp.p1=G[i];
		if (((G[i][0]).divise(aux)).getCoeff()>0 && testIn(p) && testIn(pp)) return true;
	}
	return false;
}

// inter-reduit G
void AlgoClassiquesLex::interReduction() {
	std::vector<PolPlusVarLex> tabAux;

	// 1er parcours: on reduit
	for (int i=(int)G.size()-1; i>=0; i--) {
		tabAux.clear();
		for (int j=0; j<i; j++) tabAux.push_back(G[j]);
		for (int j=i+1; j<(int)G.size(); j++) tabAux.push_back(G[j]);
		G[i]=G[i].reduction(tabAux);
	}

	// second parcours: on elimine les polynomes nuls
	tabAux.clear();
	for (int i=0; i<(int)G.size(); i++)
		if (G[i].getSize()!=0) tabAux.push_back(G[i]);
	G=tabAux;
	return;
}

// algo basique
void AlgoClassiquesLex::Buchberger() {
	PolPlusVarLex sPol, red;
	Paire paire;
	nbIter=0;
	while (!S.empty()) {
		// pour l'instant on ne trie pas les elements de S
		paire=S.top();
		S.pop();
		sPol=(paire.p1).sPoly(paire.p2);
		// remarque: 1ere partie (1) de la strategie normale utilisee pour la reduction [red par les + vieux pol first]
		red=sPol.reduction(G);
		if (red.getSize()!=0) {
			paire.p1=red;
			for (int i=G.size()-1; i>=0; i--) {
				paire.p2=G[i];
				ajoutePaire(paire);
			}
			G.push_back(red);
		}
		nbIter++;
		if (nbIter>500) {
			std::cout << "Algorithme stoppe, >=500 iterations" << std::endl;
			ordonne();
			return;
		}
	}
	// normalisation puis ordonnancement
	normalise();
	ordonne();
	return;
}

// algo avec optimisations classiques
std::vector<PolPlusVarLex> AlgoClassiquesLex::BuchOpt() {
	Paire paire;
	std::vector<PolPlusVarLex> nouv;
	PolPlusVarLex sPol, red;
	TermLex aux,tmp;
	nbIter=0;
	while (!S.empty()) {  /*std::cout << "taille de S " << S.size() << std::endl;
		std::cout << "iteration " << nbIter << std::endl << std::endl;*/
		paire=S.top();
		S.pop();
		if (testInutile(paire)) continue;
		dejaTraitees.push_back(paire);
		sPol=(paire.p1).sPoly(paire.p2);
		red=sPol.reduction(G);
		if (red.getSize()!=0) {
			nouv.clear();
			paire.p1=red;
			for (int i=G.size()-1; i>=0; i--) {
				paire.p2=G[i];
				// application de la strategie normale (2) en triant les paires dans l'ordre des degres de leurs ppcm
				ajoutePaire(paire);
			}
			aux=red[0];
			for (int i=G.size()-1; i>=0; i--) {
				tmp=aux.divise(G[i][0]);
				// on ne garde que les polynomes dont le HT n'est pas divisible par red.HT
				if (tmp.getCoeff()==0) nouv.push_back(G[i]);
			}
			nouv.push_back(red);
			G=nouv;
		}
		nbIter++;
		if (nbIter>500) {
			std::cout << "Algorithme stoppe, >=500 iterations" << std::endl;
			normalise();
			ordonne();
			return G;
		}
	}
	// Inter-reduction de G
	interReduction();

	// normalisation puis ordonnancement
	normalise();
	ordonne();

	if (G.empty()) G.push_back(PolPlusVarLex());
	return G;
}
