#ifndef _AlgoClassiquesLex
#define _AlgoClassiquesLex

#include <queue>
#include <vector>
#include <algorithm>
#include "PolPlusVarLex.h"

struct Paire {
	// constructeur vide requis par priority_queue
	Paire () {}
	bool operator<(const Paire& pa) const { // precondition: paire non nulle
		// calcul des ppcm des monomes de tete
		if ((p1[0].ppcm(p2[0])).infLex(pa.p1[0].ppcm(pa.p2[0]))) return true;
		else return false;
	}
	PolPlusVarLex p1;
	PolPlusVarLex p2;
};

class AlgoClassiquesLex {

public:

	// constructeur vide
	AlgoClassiquesLex() {}

	// constructeur: initialisation de G et S
	AlgoClassiquesLex(const std::vector<PolPlusVarLex>& tabPol);

	// fonction d'affichage de G
	void affiche();

	// affichage sous forme d'un systeme d'equations a satisfaire
	void afficheEqualZero();

	// teste l'appartenance d'une paire a dejaTraitees
	bool testIn(const Paire& paire) const {
		for (int i=dejaTraitees.size()-1; i>=0; i--)
			if (paire.p1==dejaTraitees[i].p1 && paire.p2==dejaTraitees[i].p2) return true;
		return false;
	}

	// teste si une paire est inutile ou triviale
	bool testInutile(const Paire& paire) const;

	// range une paire (non nulle) dans S (trie!) en fonction de son ppcm (ordre croissant)
	void ajoutePaire(const Paire& pr) {
		if (pr.p1.getSize()!=0 && pr.p2.getSize()!=0) S.push(pr);
		return;
	}

	// retourne le nombre d'iterations effectuees
	int getNbIter() const {
		return nbIter;
	}

	// inter-reduit G
	void interReduction();

	// actualisation des coefficients pour normaliser a 1 dans G
	void normalise() {
		for (int i=0; i<(int)G.size(); i++) {
			for (int j=1; j<G[i].getSize(); j++)
				G[i].setCoeff(j,G[i][j].getCoeff()/G[i][0].getCoeff());
			G[i].setCoeff(0,1);
		}
		return;
	}

	// ordonne G selon Lex avec un quicksort
	void ordonne() {
		sort(G.begin(),G.end());
	}

	// algo basique (a partir du contenu de G)
	void Buchberger();

	// algo avec optimisations classiques
	std::vector<PolPlusVarLex> BuchOpt();

	// destructeur
	~AlgoClassiquesLex() {}

private:

	// nombre d'iterations effectuees
	int nbIter;

	// ensemble de polynomes generateurs
	std::vector<PolPlusVarLex> G;

	// ensemble de paires de polynomes a reduire
	std::priority_queue<Paire> S;

	// ensemble de paires de polynomes deja traitees
	std::vector<Paire> dejaTraitees;

};

#endif
