/* classe pour manipuler des entiers modulo p premier (donc elements de Z/pZ) */
/* etape suivante: creer un type fraction avec des class_mpz, et travailler dans Q */

#include <iostream>

#ifndef _Fp
#define _Fp

class Fp {

public:

	// constructeurs
	Fp() {}

	Fp(int x);

	// acces a la valeur
	int getValue() const {
		return value;
	}

	// acces a p
	int getP() const {
		return p;
	}

	// redefinitions des methodes arithmetiques usuelles, dont l'inversion modulo p,
	// qui sert d'auxiliaire a la division modulaire
	bool operator==(int x) const {
		if (x<0) return false; // important pour l'affichage; a supprimer si on sort du contexte
		return value==(Fp(x)).getValue();
	}

	bool operator==(const Fp& x) const {
		return value==x.getValue();
	}

	bool operator!=(int x) const {
		if (x<0) return true; // meme remarque
		return value!=(Fp(x)).getValue();
	}

	bool operator!=(const Fp& x) const {
		return value!=x.getValue();
	}

	Fp operator-() const {
		return Fp(-value);
	}

	bool operator<(int x)  const { // ...etc (mais OK meme dans le contexte)
		return value<(Fp(x)).getValue();
	}

	bool operator>(int x) const {
		return value>(Fp(x)).getValue();
	}

	bool operator<=(int x) const {
		return value<=(Fp(x)).getValue();
	}

	bool operator>=(int x) const {
		return value>=(Fp(x)).getValue();
	}

	Fp operator+(const Fp& x) const {
		return Fp(value+x.getValue());
	}

	Fp operator-(const Fp& x) const {
		return Fp(value-x.getValue());
	}

	Fp operator*(const Fp& x) const {
		return Fp(value*x.getValue());
	}

	int invMod() const;

	Fp operator/(const Fp& x) const {
		return Fp(value*x.invMod());
	}

	// destructeur
	~Fp() {}

private:

	// 0..p-1
	int value;

	// l'entier premier utilise
	int p;

};

// affichage
extern std::ostream& operator<<(std::ostream& o, const Fp& x);

#endif
