#include "PolPlusVarGrevlex.h"

// insertion relativement a l'ordre lexicographique
void PolPlusVarGrevlex::addTerm(const TermGrevlex& term) {
	// on verifie si par hasard le terme ajoute est deja present
	std::vector<TermGrevlex>::iterator it=tabTermes.end();
	for (int i=tabTermes.size()-1; i>=0; i--) {
		if (tabTermes[i]==term) {
			Gen aux=tabTermes[i].getCoeff()+term.getCoeff();
			if (aux==0) {
				tabTermes.erase(--it);
				return;
			}
			tabTermes[i].setCoeff(aux);
			return;
		}
		if (term.infGrevlex(tabTermes[i])) {
			tabTermes.insert(it, term);
			return;
		}
		it--;
	}
	// terme non present: on l'ajoute betement
	tabTermes.insert(tabTermes.begin(), term);
	return;
}

// soustraction de deux PolPlusVarGrevlex tries
PolPlusVarGrevlex PolPlusVarGrevlex::operator-(const PolPlusVarGrevlex& pol) const { // complexite ? --> a ameliorer!!

	/* std::cout << "arg: " << std::endl;
	std::cout << *this << std::endl;
	std::cout << pol << std::endl;*/

	PolPlusVarGrevlex pol_res=PolPlusVarGrevlex();
	int i=0;
	int j=0;
	while (i<(int)tabTermes.size() && j<(int)pol.getSize()) {
		if (tabTermes[i].infGrevlex(pol[j])) {
			//std::cout << tabTermes[i] << " <= " << pol[j] << "donc j'ajoute " << pol[j] << std::endl;
			pol_res.addTerm(-pol[j]);
			//std::cout << "j'obtiens alors " << pol_res << std::endl;
			j++;
		}
		else {
			//std::cout << tabTermes[i] << " > " << pol[j] << "donc j'ajoute " << tabTermes[i] << std::endl;
			pol_res.addTerm(tabTermes[i]);
			//std::cout << "j'obtiens alors " << pol_res << std::endl;
			i++;
		}
	}
	if (i<(int)tabTermes.size()) for (j=i; j<(int)tabTermes.size(); j++) pol_res.addTerm(tabTermes[j]);
	else for (i=j; i<pol.getSize(); i++) pol_res.addTerm(-pol[i]);
	return pol_res;
}

// multiplication d'un PolPlusVarGrevlex par un terme
PolPlusVarGrevlex PolPlusVarGrevlex::operator*(const TermGrevlex& term) const { // complexite ?
	PolPlusVarGrevlex pol_res=PolPlusVarGrevlex();
	TermGrevlex aux;
	int p;
	Gen q=term.getCoeff();
	for (p=0; p<(int)tabTermes.size(); p++) {
		aux=TermGrevlex(term.getSize());
		for (int i=0; i<term.getSize(); i++)
			aux.addElt(tabTermes[p][i]+term[i]);
		aux.setCoeff(tabTermes[p].getCoeff()*q);
		pol_res.addTerm(aux);
	}
	return pol_res;
}

// calcul du S-Polynome de deux PolPlusVarGrevlex: f et g
PolPlusVarGrevlex PolPlusVarGrevlex::sPoly(const PolPlusVarGrevlex& pol) const { // complexite?
	if (tabTermes.size()==0 || pol.getSize()==0) return PolPlusVarGrevlex();
	TermGrevlex term_ppcm=tabTermes[0].ppcm(pol[0]); // calcul du ppcm des monomes de tete utile pour la suite
	TermGrevlex term1=TermGrevlex(tabTermes[0].getSize()), term2=TermGrevlex(tabTermes[0].getSize());

	// divisions du ppcm par HT(f), HT(g)
	for (int i=0; i<tabTermes[0].getSize(); i++) {
		term1.addElt(term_ppcm[i]-tabTermes[0][i]);
		term2.addElt(term_ppcm[i]-pol[0][i]);
	}

	// multiplications par HC(f), HC(g)
	term1.setCoeff(pol[0].getCoeff());
	term2.setCoeff(tabTermes[0].getCoeff());

	// multiplications par f, g puis soustraction
	PolPlusVarGrevlex pol_res=PolPlusVarGrevlex((*this)*term1-pol*term2);
	return pol_res;
}

// reduction d'un polynome par un ensemble de polynomes
PolPlusVarGrevlex PolPlusVarGrevlex::reduction(const std::vector<PolPlusVarGrevlex>& tabPol) const { // complexite ?
	bool red=true; // indique si une reduction a ete effectuee dans ce passage
	if (tabTermes.size()==0) return *this;
	PolPlusVarGrevlex pol_res=PolPlusVarGrevlex(*this);
	TermGrevlex HTi, HTres, divi;

	while (red) { // on s'arrete donc lorsqu'on fait un parcours sans reduction
		red=false;
		HTres=pol_res[0];
		for (int i=0; i<(int)tabPol.size(); i++) { // on reduit en priorite par les polynomes les plus anciens (1)
			if (tabPol[i].getSize()!=0) {
				HTi=tabPol[i][0];
				// tentative de reduction par le monome de tete
				divi=HTi.divise(HTres);
				if (divi.getCoeff()!=0) {
				divi.setCoeff(HTres.getCoeff()/HTi.getCoeff());
				pol_res=pol_res-tabPol[i]*divi;
				if (pol_res.getSize()==0) return pol_res;
					HTres=pol_res[0];
				red=true;
				}
			}
		}
	}
	return pol_res;
}

// affichage d'un PolPlusVarGrevlex
std::ostream& operator<<(std::ostream& o, const PolPlusVarGrevlex& pol) {
	if (pol.getSize()==0) {
		o << 0;
		return o;
	}
	for (int i=0; i<(int)pol.getSize(); i++) {
		o << pol[i];
		if (i<(int)pol.getSize()-1 && pol[i+1].getCoeff()>0) o << " + ";
	}
	return o;
}
