#ifndef _PolPlusVarGrevlex
#define _PolPlusVarGrevlex

#include <algorithm>
#include <vector>
#include <iostream>

#include "TermGrevlex.h"

class PolPlusVarGrevlex {

public:

	// constructeur vide
	PolPlusVarGrevlex() {}

	// constructeur a partir d'un terme
	PolPlusVarGrevlex(const TermGrevlex& term) {
		tabTermes.push_back(term);
	}

	// constructeur de recopie
	PolPlusVarGrevlex(const PolPlusVarGrevlex& pol) {
		for (int i=0; i<pol.getSize(); i++) tabTermes.push_back(pol[i]);
	}

	// redefinition de l'egalite entre deux PolPlusVarGrevlex
	bool operator==(const PolPlusVarGrevlex& pol) const {
		if ((int)tabTermes.size()!=pol.getSize()) return false;
		for (int i=tabTermes.size()-1; i>=0; i--)
			if (tabTermes[i]!=pol[i]) return false;
		return true;
	}

	// precondition: le monome ajoute a un coeff non nul, et le polynome est trie dans l'ordre grevlex
	// postcondition: le polynome courant += terme, sans termes nuls; resultat ordonne selon lex
	void addTerm(const TermGrevlex& term);

	// definition d'une methode setCoeff, comme pour TermGrevlex
	void setCoeff(int rang, int coeff) {
		tabTermes[rang].setCoeff(coeff);
	}
	void setCoeff(int rang, Gen coeff) {
		tabTermes[rang].setCoeff(coeff);
	}

	// renvoie le nombre de monomes dans le polynome
	int getSize() const {
		return tabTermes.size();
	}

	// redefinition de l'operateur []: pol[p] est le p-ieme vecteur de pol si pol est un PolPlusVarGrevlex
	TermGrevlex operator[](int p) const {
		return tabTermes[p];
	}

	// redefinition de l'operateur de comparaison suivant les monomes de tete
	bool operator<(const PolPlusVarGrevlex& pol) const {
		return tabTermes[0].infGrevlex(pol[0]);
	}

	// tri des termes
	void tri() {
		sort(tabTermes.begin(),tabTermes.end());
	}

	// soustraction de deux PolPlusVarGrevlex
	PolPlusVarGrevlex operator-(const PolPlusVarGrevlex& pol) const;

	// multiplication d'un PolPlusVarGrevlex par un terme
	PolPlusVarGrevlex operator*(const TermGrevlex& term) const;

	// calcul du S-Polynome de deux PolPlusVarGrevlex
	PolPlusVarGrevlex sPoly(const PolPlusVarGrevlex& pol) const;

	// reduction d'un polynome par un ensemble de polynomes
	PolPlusVarGrevlex reduction(const std::vector<PolPlusVarGrevlex>& tabPol) const;

	// destructeur
	~PolPlusVarGrevlex() {}

private:

	// tabTermes est un tableau de vecteurs, representant chacun un terme en les indeterminees An
	std::vector<TermGrevlex> tabTermes;

};

// affichage d'un PolPlusVarGrevlex
extern std::ostream& operator<<(std::ostream& o, const PolPlusVarGrevlex& pol);

#endif
