#include "PolPlusVarLex.h"

// insertion relativement a l'ordre lexicographique
void PolPlusVarLex::addTerm(const TermLex& term) {
	// on verifie si par hasard le terme ajoute est deja present
	std::vector<TermLex>::iterator it=tabTermes.end();
	for (int i=tabTermes.size()-1; i>=0; i--) {
		if (tabTermes[i]==term) {
			Gen aux=tabTermes[i].getCoeff()+term.getCoeff();
			if (aux==0) {
				tabTermes.erase(--it);
				return;
			}
			tabTermes[i].setCoeff(aux);
			return;
		}
		if (term.infLex(tabTermes[i])) {
			tabTermes.insert(it, term);
			return;
		}
		it--;
	}
	// terme non present et plus grand que tous les autres:
	// ajout en tete
	tabTermes.insert(tabTermes.begin(), term);
	return;
}

// soustraction de deux PolPlusVarLex tries
PolPlusVarLex PolPlusVarLex::operator-(const PolPlusVarLex& pol) const {
	PolPlusVarLex pol_res=PolPlusVarLex();
	int i=0;
	int j=0;
	while (i<(int)tabTermes.size() && j<(int)pol.getSize()) {
		if (tabTermes[i].infLex(pol[j])) {
			pol_res.addTerm(-pol[j]);
			j++;
		}
		else {
			pol_res.addTerm(tabTermes[i]);
			i++;
		}
	}
	if (i<(int)tabTermes.size())
		for (j=i; j<(int)tabTermes.size(); j++) pol_res.addTerm(tabTermes[j]);
	else for (i=j; i<pol.getSize(); i++) pol_res.addTerm(-pol[i]);
	return pol_res;
}

// addition suivant le meme schema (simplifie la multiplication plus bas)
PolPlusVarLex PolPlusVarLex::operator+(const PolPlusVarLex& pol) const {
	PolPlusVarLex pol_res=PolPlusVarLex();
	int i=0;
	int j=0;
	while (i<(int)tabTermes.size() && j<(int)pol.getSize()) {
		if (tabTermes[i].infLex(pol[j])) {
			pol_res.addTerm(pol[j]);
			j++;
		}
		else {
			pol_res.addTerm(tabTermes[i]);
			i++;
		}
	}
	if (i<(int)tabTermes.size()) for (j=i; j<(int)tabTermes.size(); j++) pol_res.addTerm(tabTermes[j]);
	else for (i=j; i<pol.getSize(); i++) pol_res.addTerm(pol[i]);
	return pol_res;
}

// multiplication d'un PolPlusVarLex (trie) par un terme
PolPlusVarLex PolPlusVarLex::operator*(const TermLex& term) const {
	PolPlusVarLex pol_res=PolPlusVarLex();
	TermLex aux;
	int p;
	Gen q=term.getCoeff();
	for (p=0; p<(int)tabTermes.size(); p++) {
		aux=TermLex(term.getSize());
		for (int i=0; i<term.getSize(); i++)
			aux.addElt(tabTermes[p][i]+term[i]);
		aux.setCoeff(tabTermes[p].getCoeff()*q);
		pol_res.addTerm(aux);
	}
	return pol_res;
}

// multiplication de deux PolPlusVar entre eux (necessaire pour la recherche de parametrisations)
// ATTENTION: seul endroit ou ne garantit rien vis-a-vis de l'ordre!
PolPlusVarLex PolPlusVarLex::operator*(const PolPlusVarLex& pol) const {
	PolPlusVarLex pol_res=PolPlusVarLex();
	for (int p=pol.getSize()-1; p>=0; p--)
		pol_res=pol_res+(*this)*pol[p];
	return pol_res;
}

// calcul du S-Polynome de deux PolPlusVarLex (tries): f et g
PolPlusVarLex PolPlusVarLex::sPoly(const PolPlusVarLex& pol) const {
	if (tabTermes.size()==0 || pol.getSize()==0) return PolPlusVarLex();
	TermLex term_ppcm=tabTermes[0].ppcm(pol[0]); // calcul du ppcm des monomes de tete utile pour la suite
	TermLex term1=TermLex(tabTermes[0].getSize()), term2=TermLex(tabTermes[0].getSize());

	// divisions du ppcm par HT(f), HT(g)
	for (int i=0; i<tabTermes[0].getSize(); i++) {
		term1.addElt(term_ppcm[i]-tabTermes[0][i]);
		term2.addElt(term_ppcm[i]-pol[0][i]);
	}

	// multiplications par HC(f), HC(g)
	term1.setCoeff(pol[0].getCoeff());
	term2.setCoeff(tabTermes[0].getCoeff());

	PolPlusVarLex pol_res=PolPlusVarLex((*this)*term1-pol*term2);
	return pol_res;
}

// reduction d'un polynome (trie) par un ensemble de polynomes
PolPlusVarLex PolPlusVarLex::reduction(const std::vector<PolPlusVarLex>& tabPol) const {

	//std::cout << std::endl << "arg: " << *this << std::endl;
	//for (int i=0; i<tabPol.size(); i++) std::cout << tabPol[i] << std::endl;

	bool red=true; // indique si une reduction a ete effectuee dans ce passage
	if (tabTermes.size()==0) return *this;
	PolPlusVarLex pol_res=PolPlusVarLex(*this);
	TermLex HTi, HTres, divi;

	while (red) { // on s'arrete donc lorsqu'on fait un parcours sans reduction
		red=false;
		HTres=pol_res[0];
		for (int i=0; i<(int)tabPol.size(); i++) { // on reduit en priorite par les polynomes les plus anciens
			if (tabPol[i].getSize()!=0) {
				HTi=tabPol[i][0];
				// tentative de reduction par le monome de tete
				divi=HTi.divise(HTres);
				if (divi.getCoeff()!=0) {
					divi.setCoeff(HTres.getCoeff()/HTi.getCoeff());

					//std::cout << std::endl << HTi << " divise " << HTres << " donc j'effectue : " << std::endl;
					//std ::cout << pol_res << " moins " << tabPol[i] << " fois " << divi << std::endl;

					pol_res=pol_res-tabPol[i]*divi;

					//std::cout << std::endl << "j'obtiens: " << pol_res << std::endl;

					if (pol_res.getSize()==0) return pol_res;
					HTres=pol_res[0];
					red=true;
				}
			}
		}
	}

	//std::cout << std::endl << "res: " << pol_res << std::endl;

	return pol_res;
}

// reste de la division euclidienne d'un polynome (trie) par un autre (algo precedent simplifie)
PolPlusVarLex PolPlusVarLex::reste(const PolPlusVarLex& pol) const {
	if (tabTermes.size()==0) return *this;
	if (pol.getSize()==0) {
		std::cerr << "Division par 0!" << std::endl;
		return *this;
	}
	PolPlusVarLex pol_res=PolPlusVarLex(*this);
	TermLex HT=pol[0], HTres=tabTermes[0], divi;
	while ((divi=HT.divise(HTres)).getCoeff()!=0) { // on s'arrete donc lorsque l'on ne peut plus reduire
		divi.setCoeff(HTres.getCoeff()/HT.getCoeff());
		pol_res=pol_res-pol*divi;
		if (pol_res.getSize()==0) return pol_res; // alors pol divise *this
		HTres=pol_res[0];
	}
	return pol_res;
}

// calcul du pgcd de deux polynomes par l'algorithme d'Euclide (pas de debordements: on est dans Fp)
PolPlusVarLex PolPlusVarLex::pgcd(const PolPlusVarLex& pol) const {
	if (tabTermes.empty() || pol.getSize()==0) return PolPlusVarLex(pol);
	PolPlusVarLex A=PolPlusVarLex(*this);
	PolPlusVarLex B=PolPlusVarLex(pol);
	if (A[0].infLex(B[0])) {
		PolPlusVarLex aux;
		while (A.getSize()>0) {
			aux=PolPlusVarLex(A);
			A=B.reste(A);
			B=aux;
		}
		// normalisation
		Gen coeff=B[0].getCoeff();
		for (int i=1; i<B.getSize(); i++) B.setCoeff(i,B[i].getCoeff()/coeff);
		B.setCoeff(0,1);
		return B;
	}
	else {
		PolPlusVarLex aux;
		while (B.getSize()>0) {
			aux=PolPlusVarLex(B);
			B=A.reste(B);
			A=aux;
		}
		// normalisation
		Gen coeff=A[0].getCoeff();
		for (int i=1; i<A.getSize(); i++) A.setCoeff(i,A[i].getCoeff()/coeff);
		A.setCoeff(0,1);
		return A;
	}
}

// substitution d'un element de Fp a une indeterminee
PolPlusVarLex PolPlusVarLex::subsFp(int elt,int indet) const {
	if (tabTermes.empty()) return PolPlusVarLex();
	PolPlusVarLex res=PolPlusVarLex();
	TermLex term;
	int p=tabTermes[0].getCoeff().getP();
	for (int i=0; i<(int)tabTermes.size(); i++) {
		term=TermLex(tabTermes[0].getSize());
		for (int j=0; j<indet; j++) term.addElt(tabTermes[i][j]);
		term.addElt(0);
		for (int j=indet+1; j<tabTermes[0].getSize(); j++) term.addElt(tabTermes[i][j]);
		if (tabTermes[i][indet]!=0) {
			if (elt!=0) term.setCoeff((tabTermes[i].getCoeff().getValue()*puiss(elt,tabTermes[i][indet],p))%p);
			else continue;
		}
		else term.setCoeff(tabTermes[i].getCoeff());
		res.addTerm(term);
	}
	res.tri(); // vital
	return res;
}

// affichage d'un PolPlusVarLex
std::ostream& operator<<(std::ostream& o, const PolPlusVarLex& pol) {
	if (pol.getSize()==0) {
		o << 0;
		return o;
	}
	for (int i=0; i<(int)pol.getSize(); i++) {
		o << pol[i];
		if (i<(int)pol.getSize()-1 && pol[i+1].getCoeff()>0) o << " + ";
	}
	return o;
}
