#ifndef _PolPlusVarLex
#define _PolPlusVarLex

#include <algorithm>
#include <vector>
#include <iostream>

#include "TermLex.h"

class PolPlusVarLex {

public:

	// constructeur vide
	PolPlusVarLex() {}

	// constructeur a partir d'un terme
	PolPlusVarLex(const TermLex& term) {
		tabTermes.push_back(term);
	}

	// constructeur de recopie
	PolPlusVarLex(const PolPlusVarLex& pol) {
		for (int i=0; i<pol.getSize(); i++) tabTermes.push_back(pol[i]);
	}

	// redefinition de l'egalite entre deux PolPlusVarLex
	bool operator==(const PolPlusVarLex& pol) const {
		if ((int)tabTermes.size()!=pol.getSize()) return false;
		for (int i=tabTermes.size()-1; i>=0; i--)
			if (tabTermes[i]!=pol[i]) return false;
		return true;
	}

	// precondition: le monome ajoute a un coeff non nul, et le polynome est trie dans l'ordre lex
	// postcondition: le polynome courant += terme, sans termes nuls; resultat ordonne selon lex
	void addTerm(const TermLex& term);

	// definition d'une methode setCoeff, comme pour TermLex
	void setCoeff(int rang, int coeff) {
		tabTermes[rang].setCoeff(coeff);
	}
	void setCoeff(int rang, Gen coeff) {
		tabTermes[rang].setCoeff(coeff);
	}

	// renvoie le nombre de monomes dans le polynome
	int getSize() const {
		return tabTermes.size();
	}

	// redefinition de l'operateur []: pol[p] est le p-ieme vecteur de pol si pol est un PolPlusVarLex
	TermLex operator[](int p) const {
		return tabTermes[p];
	}

	// redefinition de l'operateur de comparaison suivant les monomes de tete
	bool operator<(const PolPlusVarLex& pol) const {
		return tabTermes[0].infLex(pol[0]);
	}

	// tri des termes
	void tri() {
		sort(tabTermes.begin(),tabTermes.end());
	}

	// soustraction de deux PolPlusVarLex
	PolPlusVarLex operator-(const PolPlusVarLex& pol) const;

	// addition suivant le meme schema
	PolPlusVarLex operator+(const PolPlusVarLex& pol) const;

	// multiplication d'un PolPlusVarLex par un terme
	PolPlusVarLex operator*(const TermLex& term) const;

	// multiplication de deux PolPlusVar entre eux (necessaire pour la recherche de parametrisations)
	PolPlusVarLex operator*(const PolPlusVarLex& pol) const;

	// calcul du S-Polynome de deux PolPlusVarLex
	PolPlusVarLex sPoly(const PolPlusVarLex& pol) const;

	// reduction d'un polynome par un ensemble de polynomes
	PolPlusVarLex reduction(const std::vector<PolPlusVarLex>& tabPol) const;

	// division euclidienne d'un polynome (trie) par un autre (algo precedent simplifie)
	PolPlusVarLex reste(const PolPlusVarLex& pol) const;

	// calcul du pgcd de deux polynomes par l'algorithme d'euclide (pas de debordements: on est dans Fp)
	PolPlusVarLex pgcd(const PolPlusVarLex& pol) const;

	// substitution d'un element de Fp a une indeterminee
	PolPlusVarLex subsFp(int elt,int indet) const;

	// elevation a une puissance modulaire non nulle
	int puiss(int a, int b, int p) const {
		if (b<=0) return 1;
		if (b%1) return puiss(a,b/2,p);
		else return (puiss(a,b/2,p)*a)%p;
	}

	// destructeur
	~PolPlusVarLex() {}

private:

	// tabTermes est un tableau de vecteurs, representant chacun un terme en les indeterminees An
	std::vector<TermLex> tabTermes;

};

// affichage d'un PolPlusVarLex
extern std::ostream& operator<<(std::ostream& o, const PolPlusVarLex& pol);

#endif
