#include "TermGrevlex.h"

// constructeur par recopie
TermGrevlex::TermGrevlex(const TermGrevlex& term) {
	n=term.getSize();
	degTot=term.getDeg();
	coeff=term.getCoeff();
	indet=(int*)malloc(n*sizeof(int));
	for (int i=n-1; i>=0; i--) indet[i]=term[i];
}

// prend un tableau d'entiers en parametre, que l'on transforme en vecteur
TermGrevlex::TermGrevlex(int n, Gen coeff, int* tab_indet) {
	this->n=n;
	this->coeff=coeff;
	degTot=0;
	indet=(int*)malloc(n*sizeof(int));
	for (int i=0; i<n; i++) {
		indet[i]=tab_indet[i];
		degTot+=tab_indet[i];
	}
}

// redefinition de l'operateur de comparaison grevlex
bool TermGrevlex::infGrevlex(const TermGrevlex& term) const {
	if (degTot<term.getDeg()) return true;
	if (degTot>term.getDeg()) return false;
	int aux;
	for (int i=n-1; i>=0; i--) {
		aux=indet[i]-term[i];
		if (aux>0) return true;
		else if (aux<0) return false;
	}
	// cas de deux termes egaux (modulo le coefficient)
	return true;
}

// operateur ne servant que pour le tri (resultat inverse!)
bool TermGrevlex::operator<(const TermGrevlex& term) const {
	if (degTot<term.getDeg()) return false;
	if (degTot>term.getDeg()) return true;
	int aux;
	for (int i=n-1; i>=0; i--) {
		aux=indet[i]-term[i];
		if (aux>0) return false;
		else if (aux<0) return true;
	}
	// cas de deux termes egaux (modulo le coefficient)
	return true;
}

// comparaison de deux termes pour la relation de divisibilite: si la division est possible,
// on l'effectue en renvoyant son resultat; sinon on renvoie un terme vide avec pour coefficient 0.
TermGrevlex TermGrevlex::divise(const TermGrevlex& term) const {
	int* liste=(int*)malloc(n*sizeof(int));
	for (int i=n-1; i>=0; i--) {
		if (indet[i]>term[i]) { // alors le terme courant ne divise pas l'autre
			TermGrevlex term_res=TermGrevlex(n,0,liste);
			term_res.setCoeff(0);
			return term_res;
		}
		else *(liste+i)=term[i]-indet[i];
	}
	// si on arrive ici, la division est possible
	TermGrevlex term_res=TermGrevlex(n,1,liste);
	return term_res;
}

// affichage d'un TermGrevlex
std::ostream& operator<<(std::ostream& o, const TermGrevlex& term) {
	int n=term.getSize();
	Gen coeff=term.getCoeff();
	if (coeff==0) {
		o << 0;
		return o;
	}
	int j=0;
	while (term[j]==0 && j<n) j++;
	if (j==n) {
		o << coeff;
		return o;
	}
	if (coeff!=1 && coeff!=-1) {
		if (coeff<0)
			o << " " << coeff;
		else o << coeff;
		for (; j<n; j++) {
			if (term[j]!=0) {
				o << "*" << "X" << j+1;
				if (term[j]!=1)
				o << "^" << term[j];
			}
		}
	}
	else if (coeff==-1) {
		o << " - X" << j+1;
		if (term[j]!=1)
			o << "^" << term[j];
		j++;
		for (; j<n; j++) {
			if (term[j]!=0) {
				o << "*" << "X" << j+1;
				if (term[j]!=1)
				o << "^" << term[j];
			}
		}
	}
	else {
		o << "X" << j+1;
		if (term[j]!=1)
			o << "^" << term[j];
		j++;
		for (; j<n; j++) {
			if (term[j]!=0) {
				o << "*" << "X" << j+1;
				if (term[j]!=1)
					o << "^" << term[j];
			}
		}
	}
	return o;
}
