#ifndef _TermLex
#define _TermLex

#include <iostream>
#include <vector>
#include <cstdlib>

#include "Fp.h"
//#include "Something_More_General.h"

// type du coefficient
#define Gen Fp // entier modulo p
//#define Gen Something_More_General...

class TermLex {

public:

	// constructeurs
	TermLex() {
		indet = NULL;
	}

	// on donne juste le nombre d'indeterminees
	TermLex(int nb) {
		degTot=0;
		n=nb;
		indice=0;
		indet=(int*)malloc(n*sizeof(int));
	}

	// constructeur par recopie
	TermLex(const TermLex& term);

	// prend un tableau d'entiers en parametre, que l'on transforme en vecteur
	TermLex(int n, Gen coeff, int* tab_indet); // size est la taille du tableau

	// redefinition de l'egalite (limitee aux puissances des indeterminees) entre deux termes
	bool operator==(const TermLex& term) const {
		for (int i=n-1; i>=0; i--)
			if (term[i]!=indet[i]) return false;
		return true;
	}

	// redefinition de la "non egalite" de deux termes
	bool operator!=(const TermLex& term) const {
		if (coeff!=term.getCoeff()) return true;
		for (int i=n-1; i>=0; i--)
			if (term[i]!=indet[i]) return true;
		return false;
	}

	// transformation d'un terme en son oppose
	TermLex operator-() const {
		TermLex res=TermLex(*this);
		res.setCoeff(-coeff);
		return res;
	}

	// definition des ordres utilisables
	bool infLex(const TermLex& term) const;
	bool operator<(const TermLex& term) const; // en fait >= (!)

	// ajout du coefficient
	void setCoeff(int coeff) {
		this->coeff=Gen(coeff);
	}
	void setCoeff(Gen coeff) {
		this->coeff=coeff;
	}

	// ajoute une puissance en fin du tableau
	void addElt(int var) {
		degTot+=var;
		indet[indice]=var;
		indice++;
	}

	// acces au degre total
	int getDeg() const {
		return degTot;
	}

	// acces aux exposants
	int operator[](int p) const {
		return indet[p];
	}

	// acces a la taille du vecteur (==n)
	int getSize() const {
		return n;
	}

	// acces au coefficient
	Gen getCoeff() const {
		return coeff;
	}

	// calcul du PPCM de deux termes de meme taille (ie meme nb d'indeterminees==n, meme si les puissances sont nulles)
	TermLex ppcm(const TermLex& term) const {
		TermLex term_res=TermLex(n);
		term_res.setCoeff(1);
		for (int i=0; i<n; i++)
			if (indet[i]>term[i]) term_res.addElt(indet[i]);
			else term_res.addElt(term[i]);
		return term_res;
	}

	// teste si deux monomes sont premiers entre eux
	bool prem(const TermLex& term) const {
		for (int i=n-1; i>=0; i--)
			if (indet[i]!=0 && term[i]!=0) return false;
		return true;
	}

	// comparaison de deux termes pour la relation de divisibilite: si la division est possible,
	// on l'effectue en renvoyant son resultat; sinon on renvoie un terme vide avec pour coefficient 0.
	TermLex divise(const TermLex& term) const;

	// destructeur
	~TermLex() {
		if (indet!=NULL) free(indet);
	}

private:

	// coefficient
	Gen coeff;

	// puissances des indeterminees
	int* indet;

	// n est le nombre d'indeterminees utilisees
	int n;

	// indice courant de remplissage
	int indice;

	// degre total (pour eviter un recalcul)
	int degTot;

};

// affichage d'un TermLex
extern std::ostream& operator<<(std::ostream& o, const TermLex& term);

#endif
