/* mise sous forme implicite d'une parametrisation */

#include "Fp.h"
#include "TermLex.h"
#include "PolPlusVarLex.h"
#include "DecodePolynome.h"
#include "AlgoClassiquesLex.h"
#include <iostream>
#include <vector>

using namespace std;

int main(int argc, char **argv) {
	vector<PolPlusVarLex> numerateurs;
	vector<PolPlusVarLex> denominateurs;
	int nbQuo=0; // nombre de quotients rencontres

	// On lit les fractions [0->n] en ligne de commande, les barres de fractions sont '/' et les separations sont les espaces
	// a l'exception de la fraction nulle (elles doivent etre toutes indiquees). exemple: X / Y 0 Y / Z
	// dans le cas d'un fichier: une fraction est alors constituee de deux lignes, numerateur puis denominateur,
	// sauf dans le cas d'une fraction nulle: un seul 0.

	// test pour savoir si l'utilisateur a rentre un fichier ou des polynomes en ligne de commande
	std::ifstream file(argv[1]);
	if (file!=0) {
		// on lit dans un fichier
		std::vector<PolPlusVarLex> tabPol=readFileLex(argv[1]);
		int i=0;
		while (i<(int)tabPol.size()) {
			if (tabPol[i].getSize()==0) i++;
			else {
				numerateurs.push_back(tabPol[i]);
				denominateurs.push_back(tabPol[++i]);
				i++; // on se place sur le quotient suivant (s'il existe!)
				nbQuo++; // on a un quotient supplementaire
			}
		}
	}
	else {
		// on lit la ligne de commande
		int i=1;
		while (i<argc) {
			if (argv[i][0]=='0') i+=2;
			else {
				numerateurs.push_back(str2PolLex(argv[i]));
				i++;
				denominateurs.push_back(str2PolLex(argv[i]));
				i++; // on se place sur le quotient suivant (s'il existe!)
				nbQuo++; // on a un quotient supplementaire
			}
		}
	}

	int n=nbQuo; // le nombre de relations donnees
	// on applique l'algorithme [ordre U,X1..Xm Xm+1..Xm+n ou m est le nombre d'indeterminees donnees et n=nbIndet
	// calcul des bases de Groebner des ideaux engendres par les polynomes den(i)*Xi-num(i) et Uden(1)...den(n)

	// il faut eliminer U et les variables donnes par l'utilisateur, donc on utilise l'ordre lexicographique
	// lex(U,T1,..,Tm,X1,..,Xn) ou X1..Xn sont les indeterminees que l'on veut garder

	// on commence donc par augmenter la taille des polynomes rentres de n+1 (P(T1,..,Tm) devient P'(U,T1,..,Tm,X1,..,Xn)
	// puis on n'a plus qu'a calculer la base de Groebner.

	// a) calcul du nombre maximum d'indeterminees rencontrees dans les num/den
	int m=0;
	for (int i=n-1; i>=0; i--) {
		if (numerateurs[i].getSize()>0 && numerateurs[i][0].getSize()>m) m=numerateurs[i][0].getSize();
		if (denominateurs[i].getSize()>0 && denominateurs[i][0].getSize()>m) m=denominateurs[i][0].getSize();
	}

	// b) mise a jour des polynomes pour que tous les monomes soient de taille m
	for (int i=n-1; i>=0; i--) {
		if (numerateurs[i].getSize()>0 && numerateurs[i][0].getSize()<m) {
			PolPlusVarLex numer=PolPlusVarLex();
			for (int j=0; j<numerateurs[i].getSize(); j++) {
				TermLex term=TermLex(m);
				term.setCoeff(numerateurs[i][j].getCoeff());
				for (int k=0; k<numerateurs[i][0].getSize(); k++)
				term.addElt(numerateurs[i][j][k]);
				for (int k=numerateurs[i][0].getSize(); k<m; k++)
				term.addElt(0);
				numer.addTerm(term);
			}
			numerateurs[i]=numer;
		}
		if (denominateurs[i].getSize()>0 && denominateurs[i][0].getSize()<m) {
			PolPlusVarLex denom=PolPlusVarLex();
			for (int j=0; j<denominateurs[i].getSize(); j++) {
				TermLex term=TermLex(m);
				term.setCoeff(denominateurs[i][j].getCoeff());
				for (int k=0; k<denominateurs[i][0].getSize(); k++)
				term.addElt(denominateurs[i][j][k]);
				for (int k=denominateurs[i][0].getSize(); k<m; k++)
				term.addElt(0);
				denom.addTerm(term);
			}
			denominateurs[i]=denom;
		}
	}

	// c) mise a jour des polynomes pour que tous les monomes soient de taille n+m+1
	for (int i=n-1; i>=0; i--) {
		PolPlusVarLex numer=PolPlusVarLex();
		for (int j=0; j<numerateurs[i].getSize(); j++) {
			TermLex term=TermLex(n+m+1);
			term.setCoeff(numerateurs[i][j].getCoeff());
			term.addElt(0); // l'indeterminee U
			for (int k=0; k<m; k++)
			term.addElt(numerateurs[i][j][k]);
			for (int k=n-1; k>=0; k--)
			term.addElt(0); // les indeterminees X1..Xn en partie gauche
			numer.addTerm(term);
		}
		numerateurs[i]=numer;
		PolPlusVarLex denom=PolPlusVarLex();
		for (int j=0; j<denominateurs[i].getSize(); j++) {
			TermLex term=TermLex(n+m+1);
			term.setCoeff(denominateurs[i][j].getCoeff());
			term.addElt(0); // l'indeterminee U
			for (int k=0; k<m; k++)
			term.addElt(denominateurs[i][j][k]);
			for (int k=n-1; k>=0; k--) term.addElt(0);
			denom.addTerm(term);
		}
		denominateurs[i]=denom;
	}

	// d) copie de sauvegarde des denominateurs
	vector<PolPlusVarLex> anciensDen;
	for (int i=n-1; i>=0; i--) anciensDen.push_back(denominateurs[i]);

	// e) actualisation des denominateurs Di<-Di*Xi
	TermLex x_i;
	PolPlusVarLex pol;
	for (int i=0; i<n; i++) {
		pol=PolPlusVarLex();
		for (int j=0; j<denominateurs[i].getSize(); j++) {
			x_i=TermLex(n+m+1);
			x_i.setCoeff(denominateurs[i][j].getCoeff());
			for (int k=0; k<m+1+i; k++) x_i.addElt(denominateurs[i][j][k]);
			x_i.addElt(1);
			for (int k=m+2+i; k<m+n+1; k++) x_i.addElt(0);
			pol.addTerm(x_i);
		}
		denominateurs[i]=pol;
	}

	// f) initialisation des polynomes necessaires au calcul
	vector<PolPlusVarLex> equations;
	for (int i=n-1; i>=0; i--)
	equations.push_back(denominateurs[i]-numerateurs[i]);
	// produit des denominateurs sauvegardes
	PolPlusVarLex produitDen=anciensDen[n-1];
	for (int i=n-2; i>=0; i--) produitDen=anciensDen[i]*produitDen;
	// ajout de U dans le produit
	TermLex term=TermLex(n+m+1);
	term.setCoeff(1);
	term.addElt(1);
	for (int i=n+m-1; i>=0; i--) term.addElt(0);
	produitDen=produitDen*term;
	TermLex moinsUn=TermLex(n+m+1);
	moinsUn.setCoeff(-1);
	for (int i=n+m; i>=0; i--) moinsUn.addElt(0);
	produitDen.addTerm(moinsUn);
	// tri
	produitDen.tri();
	equations.push_back(produitDen);

	// g) calcul
	AlgoClassiquesLex bdg=AlgoClassiquesLex(equations);
	std::vector<PolPlusVarLex> tabPol=bdg.BuchOpt();

	// h) suppression des polynomes faisant intervenir U ou T1..Tm
	std::vector<PolPlusVarLex> res_aux;
	for (int i=tabPol.size()-1; i>=0; i--) {
		int j;
		for (j=tabPol[i].getSize()-1; j>=0; j--) {
			int k;
			for (k=m; k>=0; k--)
			if (tabPol[i][j][k]!=0) break;
			if (k>=0) break;
		}
		if (j<0) res_aux.push_back(tabPol[i]);
	}

	// i) "shift" des indeterminees pour ramener m-->n+m a 0-->n
	std::vector<PolPlusVarLex> res;
	for (int i=res_aux.size()-1; i>=0; i--) {
		PolPlusVarLex pol=PolPlusVarLex();
		for (int j=0; j<res_aux[i].getSize(); j++) {
			TermLex term=TermLex(n);
			term.setCoeff(res_aux[i][j].getCoeff());
			for (int k=m+1; k<=n+m; k++) term.addElt(res_aux[i][j][k]);
			pol.addTerm(term);
		}
		res.push_back(pol);
	}

	// j) affichage du resultat (ordonne selon lex)
	cout << "Meilleure relation definissant la parametrisation:" << endl;
	for (int i=res.size()-1; i>=0; i--)
		std::cout << res[i] << " = 0" << std::endl;
	if (res.empty()) cout << "0" << std::endl;

	return 0;
}
